@@ -6,6 +6,7 @@ import requests
 from bs4 import BeautifulSoup
 import pandas as pd
 import numpy as np
+# import polars as pl
 
 from .equity_curves import *
 from .signals import *
@@ -49,6 +50,7 @@ def path_reference_code(save_name):
     reference_code = str(zlib.crc32(bytes(save_name, 'UTF-8')))[:6]
     return reference_code
 
+
 def generate_filepath(para_combination, folder=''):
 
     file_format = para_combination['file_format']
@@ -66,17 +68,27 @@ def generate_filepath(para_combination, folder=''):
 
     return filepath
 
-# def read_data(path):
-
-
 
 def mp_cal_performance(tuple_data):
     para_combination = tuple_data[0]
     manager_list = tuple_data[1]
 
     result = cal_performance(para_combination)
-    index = para_combination['reference_index']
-    manager_list.append((index,result))
+
+    # new
+    para_df_dict = {}
+
+    para_df_dict['reference_code'] = path_reference_code(filename_only(para_combination))
+    para_df_dict['reference_index'] = para_combination['reference_index']
+
+    keys_to_keep = para_combination['para_dict'].keys()
+    para_df_dict.update({k: v for k, v in para_combination.items() if k in keys_to_keep})
+    para_df_dict.update(result)
+    manager_list.append(para_df_dict)
+
+
+def reference_code_apply(row):
+    return path_reference_code(filename_only(reference_code_apply.all_para_combination[row.reference_index]))
 
 
 def generate_backtest_result(all_para_combination, number_of_core=8, risk_free_rate='geometric_mean'):
@@ -100,7 +112,6 @@ def generate_backtest_result(all_para_combination, number_of_core=8, risk_free_r
 
     print('Backtest result is loading. Please wait patiently.')
 
-
     manager_list = mp.Manager().list()# To save the result with index number
 
     cal_performance_list = []
@@ -112,32 +123,7 @@ def generate_backtest_result(all_para_combination, number_of_core=8, risk_free_r
     pool.map(mp_cal_performance, cal_performance_list)
     pool.close()
 
-    df_backtest_result = pd.DataFrame()
-    df_backtest_result['reference_code'] = np.NaN
-    df_backtest_result['reference_index'] = np.NaN
-    para_keys = all_para_combination[0]['para_dict'].keys()
-    for key in para_keys:
-        df_backtest_result[key] = np.NaN
-    for key in manager_list[0][1].keys():
-        df_backtest_result[key] = np.NaN
-
-    for mp_data in manager_list:
-        reference_index = mp_data[0]
-        result_dict = mp_data[1]
-        para_combination = all_para_combination[reference_index]
-        save_name = filename_only(para_combination)
-        reference_code = path_reference_code(save_name)
-        df_backtest_result.at[reference_index, 'reference_code'] = reference_code
-        df_backtest_result.at[reference_index, 'reference_index'] = reference_index
-        for key in para_keys:
-            df_backtest_result.at[reference_index, key] = para_combination[key]
-        for key in result_dict:
-            df_backtest_result.at[reference_index, key] = result_dict[key]
-
-    # print(df_backtest_result)
-    df_backtest_result['reference_index'] = df_backtest_result['reference_index'].astype(int)
-    df_backtest_result.to_csv('backtest_result.csv')
-
+    pd.DataFrame(list(manager_list)).to_csv('backtest_result.csv')
 
 
 def plot_signal_analysis(py_filename, output_folder, start_date, end_date, para_dict, signal_settings):
@@ -296,8 +282,14 @@ def resample_summary_to_daily(para_combination, folder=''):
     freq = para_combination['freq']
     file_format = para_combination['file_format']
     sec_profile = para_combination['sec_profile']
-    margin_req = sec_profile['margin_req']
-    multiplier = sec_profile['multiplier']
+
+
+    if sectype == 'FUT':
+        margin_req = sec_profile['margin_req']
+        multiplier = sec_profile['multiplier']
+    elif sectype == 'STK':
+        multiplier = 1
+
 
     # Read backtest data
     save_path = generate_filepath(para_combination=para_combination,folder=folder)
@@ -481,6 +473,7 @@ def resample_summary_to_daily(para_combination, folder=''):
 
 
 def cal_performance(para_combination):
+
     start_date = para_combination['start_date']
     end_date = para_combination['end_date']
 
@@ -507,6 +500,14 @@ def cal_performance(para_combination):
 
     df = df_daily.copy()
 
+
+
+    # end_time = time.time()
+    # elapsed_time = end_time - start_time
+    # print("Resample: {:.2f} seconds".format(elapsed_time))
+
+
+
     # Deter if equity_value unchange = no tade
     equity_value_column = df['equity_value'].to_numpy()
     no_trade = (equity_value_column[0] == equity_value_column).all()
@@ -633,6 +634,12 @@ def cal_performance(para_combination):
 
 
 
+    # end_time = time.time()
+    # elapsed_time = end_time - start_time
+    # print("No Trade: {:.2f} seconds".format(elapsed_time))
+
+
+
     result_dict['risk_free_rate'] = risk_free_rate
 
     # BaH Performance
