@@ -583,8 +583,7 @@ class Plot:
                     df_all_para_combination.loc[df_all_para_combination['reference_index'] == reference_index].to_dict(
                         'records')[0]
 
-                    file_format = para_combination['file_format']
-                    save_path = plotguy.generate_filepath(para_combination=para_combination, file_format=file_format)
+                    save_path = plotguy.generate_filepath(para_combination=para_combination)
                     ref_code = self.graph_df.iloc[i].reference_code
                     risk_free_rate = self.graph_df.iloc[i].risk_free_rate
 
